<?php
namespace Teknisa\PED\Modules\Pedidos\Controllers;

use Teknisa\PED\Controllers\TeknisaCrud;
use Teknisa\PED\Modules\Pedidos\Services\Ped03500TabelaMetaComissaoService;
use \Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Controllers\CrudController;
use Zeedhi\Framework\DataSource\ParameterBag;
use \Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;


class Ped03500TabelaMetaComissaoController extends TeknisaCrud {

    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $TabelaMetaComissaoService;

    public function __construct($crudService, ParameterBag $parameterBag, $TabelaMetaComissaoService, $entityManager) {
        $this->TabelaMetaComissaoService = $TabelaMetaComissaoService;
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }

    public function findProduto($request, $response){
        $this->dataSourceName = 'produto_tabela_meta_comissao';
        parent::find($request,$response);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {

        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;

        if($dataSource === 'itcomissao_produto') {
            $this->parameterBagColumns = array("CDFILIAL", "CDCOMISSAO", "CDPRODUTO");
        }
        if($dataSource === 'prodmetavend_produto') {
            $this->parameterBagColumns = array("CDMETAVENDA");
        }
        parent::find($request, $response);
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try{
            $rota = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
            $data = explode("_", $rota);
            $dataSourceNameToSave = $data[0];
            $this->dataSourceNameToSave = $dataSourceNameToSave;

            parent::save($request, $response);
        }
        catch(\Exception $e) {
            throw $e;
        }
    }

    public function getNrSeq(DTO\Request $request, DTO\Response $response)  {
        $row  = $request->getRow();
        $nrseq = $row[0];
        $nrseq = $this->TabelaMetaComissaoService->geraNrSeq($nrseq);
        $dataSet = new DataSet ('data',$nrseq);
        $response->addDataSet($dataSet);
    }

    public function cadastroAutomaticoItem(DTO\Request $request, DTO\Response $response) {
        try {
            $rota = substr($request->getRoutePath(), 1);
            $data = explode("_", $rota);
            $dataSource = $data[0];
            $this->dataSourceName = $dataSource;
            $row = $request->getRow()['ROW'];
            $produtos = $request->getRow()['PRODUTOS'];

            $this->TabelaMetaComissaoService->cadastroAutomaticoItem($produtos, $row);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function cadastroAutomaticoProduto(DTO\Request $request, DTO\Response $response) {
        try{
            $rota = substr($request->getRoutePath(), 1);
            $data = explode("_", $rota);
            $dataSource = $data[0];
            $this->dataSourceName = $dataSource;
            $row = $request->getRow();

            $this->TabelaMetaComissaoService->cadastroAutomaticoProduto($row);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        try {
            $this->entityManager->beginTransaction();

            $rota = substr($request->getRoutePath(), 1);
            $data = explode("/", $rota);
            $dataSource = $data[0];

            if ($dataSource === 'metavenda') {
                $cdmetavenda = $request->getRow()['CDMETAVENDA'];
                $dataSourceDelete = 'prodmetavend';
                $conditions = array(
                    array(
            	        "value" => $cdmetavenda,
            	        "operator" => "=",
            	        "columnName" => "CDMETAVENDA"
    	            )
    	        );
                $tpOperRelacionados = new DTO\Response();
                $filterCriteria = new FilterCriteria('metavendoperac', $conditions);
                $this->crudService->find($tpOperRelacionados, $filterCriteria);
                if (count($tpOperRelacionados->getDataSets()[0]->getRows()) > 0 ) {
                    throw new \Exception ("Exclusão não permitida. Existem tipos de operações relacionados a esta meta de venda.");
                }

                $prodRelacionados = new DTO\Response();
                $filterCriteria = new FilterCriteria('prodmetavend', $conditions);
                $this->crudService->find($prodRelacionados, $filterCriteria);

                if (count($prodRelacionados->getDataSets()[0]->getRows()) > 0 ) {
                    $prodsToDelete = new DataSet ("data", $prodRelacionados->getDataSets()[0]->getRows());
                    $prodsToDelete = new DTO\Request\dataSet($prodsToDelete, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    $data = $this->dataSourceName;
                    $this->dataSourceName = 'prodmetavend';
                    parent::delete($prodsToDelete, $response);
                    $this->dataSourceName = $data;
                }
            } else if ($dataSource === 'comissao') {
                $cdcomissao = $request->getRow()['CDCOMISSAO'];
                $conditions =  array(
                    array(
            	        "value" => $cdcomissao,
            	        "operator" => "=",
            	        "columnName" => "CDCOMISSAO"
                    )
    	        );
                $itensFilhos = new DTO\Response();
                $filterCriteria = new FilterCriteria('itcomissao', $conditions);
                $this->crudService->find($itensFilhos, $filterCriteria);
                if (count($itensFilhos->getDataSets()[0]->getRows()) > 0 ) {
                    throw new \Exception ("Exclusão não permitida. Comissão não pode ser excluida.");
                }

                $vendRelacionados = new DTO\Response();
                $filterCriteria = new FilterCriteria('vendedor', $conditions);
                $this->crudService->find($vendRelacionados, $filterCriteria);
                if (count($vendRelacionados->getDataSets()[0]->getRows()) > 0 ) {
                    throw new \Exception ("Exclusão não permitida. Comissão relacionada a vendedores.");
                }
            }
            $this->dataSourceName = $dataSource;
            parent::delete($request, $response);

            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}