<?php
namespace Teknisa\PED\Services;


use Doctrine\ORM\EntityManager;
use Geral\Entities\Prazopgtclie;
use Setforms2\Shared\Registry;

class Ger00700ClienteService {
    protected $environment;
    protected $ger00700ClienteFactory;

    public function __construct($environment, $ger00700ClienteFactory) {
        $this->environment = $environment;
        $this->ger00700ClienteFactory = $ger00700ClienteFactory;
    }

    public function checkPendencias($rows) {
        
        $params = array(
                
                'P_CDCLIENTE'   => $rows['CDCLIENTE'],
                
                'P_CDFILIAL'    => $rows['CDFILIAL']
            
            );
            
        return $this->ger00700ClienteFactory->checkPendencias($params);
    
    }

    public function deleteFormaPagamento ($row) {
        $nrorg = $this->environment->getNrOrgTrab();
        try {
            $prazosPagamento = Prazopgtclie::getObjectsByWhere('NRORG = ? AND CDCLIENTE = ?', array($nrorg, $row["CDCLIENTE"]));

            foreach($prazosPagamento as $prazo) {
                $prazo->delete();
                Registry::getEntityManager()->flush();
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function rtf2Html ($rows){
        try {
            for($i=0; $i<count($rows); $i++) {
                
                $obs      = $rows[$i]['DSOBSERVCLIE'];
                $obs_clob = $rows[$i]['DSOBSERVCLIE_CLOB'];
                
                if(!$obs_clob && $obs){
                    $note = new \JoshRibakoff_Note;
                    $note->setRTF($obs);
                    $html = $note->formatHTML();
                    
                    $html = $this->setTamanhoHtml($html, $obs);
                    $rows[$i]['DSOBSERVCLIE_CLOB'] = $html;
                }
                
                /**
                 * Alterado para atender o cliente TecGeo
                 *
                 * Campo DSOBSERVCLIE (Long Raw) estourando quando atinge 4000 caracteres;
                 * 
                 * Foi necessário tirar a integração Delphi x Mobile para funcionar;
                 * 
                 * Basicamente o Mobile lê o que existe no Delphi e converte para HTML
                 * uma vez salvo, ambos não integram mais;
                 */
                /*$rtf = $rows[$i]['DSOBSERVCLIE'];
                
                if($rtf){
                    // Caso seja um HTML não precisa fazer a conversão
                    if(substr($rtf,0,1) !=='<'){
                        
                        $note = new \JoshRibakoff_Note;
                        $note->setRTF($rtf);
                        $html = $note->formatHTML();
                        
                        $html = $this->setTamanhoHtml($html, $rtf);
                        $rows[$i]['DSOBSERVCLIE'] = $html ;    
                    }       
                }else{
                    $rows[$i]['DSOBSERVCLIE'] = $rows[$i]['DSOBSERVCLIE_CLOB']? $rows[$i]['DSOBSERVCLIE_CLOB']:null;
                }*/
                
            };
            return $rows;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*
        A biblioteca não realiza a conversão de maneira adequada, sendo que algumas 
        formatações ficam perdidas.Então tentamos fazer que o tamanho
        da fonte seja mantido durante a conversão;
    */
    public function setTamanhoHtml ($html, $rtf){
        $tamanhofontes = $this->retornarTamnhosFonte($rtf);
        // caso o html possua a tag span
        if(strstr($html, "<span")){
            if(!empty($tamanhofontes)){
                $textoHtml     = '';
                $ultimoTamanho = end($tamanhofontes);
                $tagsSpan  = $this->retornaTagSpan($html);
                for($i=0; $i< count($tagsSpan); $i++){
                    // Verifica se o tamanho corresponde a tag span encontra no array dos tamnahos.Se não pega o utlimo tamanho.
                    $tamanho    = isset($tamanhofontes[$i+1]) ? $tamanhofontes[$i] : $ultimoTamanho;
                    
                    $pos        = strpos($tagsSpan[$i], 'style="');
                    $textoHtml .= substr($tagsSpan[$i],0,$pos+7);
                    $textoHtml .= 'font-size:'.$tamanho.'px;';
                    $textoHtml .= substr($tagsSpan[$i],$pos+7);
                }
                return $textoHtml;
            }else{
                return $html;
            }
        }else{
            return '<span style ="font-size:'.$tamanhofontes[0].'px">'.$html.'</span>';
            
        }
    }
    
    /* Função que percorre o texto rtf, buscnado todos os tamnhos de fontes que existem nele.
    O token responsavel pelo tamanho da fonte é \fs. */
    public function retornarTamnhosFonte ($rtf){
        $tamanhosFontes     = array();
        $numero_ocorrencias = 0;
        while( ($numero_ocorrencias = strpos($rtf, '\fs', $numero_ocorrencias+1) ) != 0)
        {
            $string       = substr($rtf,$numero_ocorrencias+3);
            $terminoFonte = strpos($string, " ");
            
            //Dividi por 2 para encontrar o tamanho real em html
            array_push($tamanhosFontes, strval (floatval(substr($string,0,$terminoFonte))/2)) ;
        }
        return $tamanhosFontes;
        
    }
    
    //Retorna todas as tags span presente na HTML
    public function retornaTagSpan ($html){
        $tagsSpan = array();
        $abreTag  = array();
        $fechaTag = array();
        $offset   = 0;
        
        while ( ($pos = strpos($html, '<span', $offset)) !== false) {
            
            $abreTag [] = $pos;
            $offset     = $pos+1;
        }
        $offset = 0;
        while ( ($pos = strpos($html, '</span>', $offset)) !== false) {
            
            $fechaTag [] = $pos;
            $offset      = $pos+1;
        }
        
        for($i=0; $i<count($abreTag); $i++){
            $tagsSpan [] = substr ($html,$abreTag[$i], $fechaTag[$i]+7);
        }
        return $tagsSpan; 
    }
    
}